import pysoem
import threading
import time
import ctypes
import struct
import tkinter as tk
from tkinter import ttk
from tkinter import messagebox

# EtherCAT 마스터 생성
master = pysoem.Master()

master.open('\\Device\\NPF_{60FC8210-8A23-4375-B5F6-31D0E27AB1ED}')

# 슬레이브 초기화
if master.config_init() > 0:
    print(f'슬레이브 {len(master.slaves)}개 검색됨!')
else:
    print('No slaves found')
    master.close()
    raise Exception("NO SLAVE")

slave = master.slaves[0] #마스터랑 먼쪽
slave1 = master.slaves[1] #마스터랑 가까운쪽

master.config_map() #슬레이브가 pre-op로 전환됨

if master.state_check(pysoem.SAFEOP_STATE, timeout=50_000) != pysoem.SAFEOP_STATE:
    master.close()
    raise Exception("not all slaves reached SAFEOP state")
        
slave.dc_sync(act=True, sync0_cycle_time=10_000_000)
slave1.dc_sync(act=True, sync0_cycle_time=10_000_000)
master.state = pysoem.OP_STATE

master.send_processdata()
master.receive_processdata(timeout=2000)

# request OP state for all slaves
master.write_state()
all_slaves_reached_op_state = False
for i in range(40):
    master.state_check(pysoem.OP_STATE, timeout=50_000)
    if master.state == pysoem.OP_STATE:
        all_slaves_reached_op_state = True
        break

if all_slaves_reached_op_state:
    print("모든 슬레이브 활성상태")

#OUTPUT PDO의 길이만큼 버퍼를 생성함
output_len1 = len(slave.output) #32
tmp1 = bytearray([0 for i in range(output_len1)])
output_len2 = len(slave1.output) #32
tmp2 = bytearray([0 for i in range(output_len2)])


# 메인 윈도우 생성
root = tk.Tk()
root.title("Tkinter Form")
root.geometry("800x500")
root.configure(bg="white")

title_label = tk.Label(root, text="녹칸다전자 이더캣 마스터 프로그램", font=("Helvetica", 30, "bold"))
title_label.pack(pady=10)

# GroupBox1
groupbox1 = ttk.LabelFrame(root, text="\uC2AC\uB808\uC774\uBE0C0\uB370\uC774\uD130", style="Custom.TLabelframe")
groupbox1.place(x=22, y=113, width=351, height=318)

labels1 = []
textboxes1 = []
for i in range(6):
    label = ttk.Label(groupbox1, text=f"Data{i+1}:", font=("\uAD81\uB9BC", 18))
    label.place(x=21, y=20 + i * 40)
    labels1.append(label)

    entry = ttk.Entry(groupbox1, font=("\uAD81\uB9BC", 18))
    entry.place(x=106, y=20 + i * 40, width=220)
    textboxes1.append(entry)

# GroupBox2
groupbox2 = ttk.LabelFrame(root, text="\uC2AC\uB808\uC774\uBE0C1\uB370\uC774\uD130", style="Custom.TLabelframe")
groupbox2.place(x=397, y=113, width=351, height=318)

labels2 = []
textboxes2 = []
for i in range(6):
    label = ttk.Label(groupbox2, text=f"Data{i+1}:", font=("\uAD81\uB9BC", 18))
    label.place(x=21, y=20 + i * 40)
    labels2.append(label)

    entry = ttk.Entry(groupbox2, font=("\uAD81\uB9BC", 18))
    entry.place(x=106, y=20 + i * 40, width=220)
    textboxes2.append(entry)

# 스타일 적용
style = ttk.Style()
style.configure("Custom.TLabelframe", font=("\uAD81\uB9BC", 18))
style.configure("Custom.TLabelframe.Label", font=("\uAD81\uB9BC", 18))

def thread1():
    while True:
        master.send_processdata()
        master.receive_processdata(timeout=2000)
        slave0_data1 = int.from_bytes(slave.input[0:4], byteorder='little', signed=True) 
        slave0_data2 = int.from_bytes(slave.input[4:8], byteorder='little', signed=False) 
        slave0_data3 = int.from_bytes(slave.input[8:10], byteorder='little', signed=True) 
        slave0_data4 = int.from_bytes(slave.input[10:12], byteorder='little', signed=False) 
        slave0_data5 = ctypes.c_int8(slave.input[12]).value
        slave0_data6 = slave.input[13]
        textboxes1[0].delete(0, 'end')
        textboxes1[0].insert(0, slave0_data1)
        textboxes1[1].delete(0, 'end')
        textboxes1[1].insert(0, slave0_data2)
        textboxes1[2].delete(0, 'end')
        textboxes1[2].insert(0, slave0_data3)
        textboxes1[3].delete(0, 'end')
        textboxes1[3].insert(0, slave0_data4)
        textboxes1[4].delete(0, 'end')
        textboxes1[4].insert(0, slave0_data5)
        textboxes1[5].delete(0, 'end')
        textboxes1[5].insert(0, slave0_data6)

        slave1_data1 = int.from_bytes(slave1.input[0:4], byteorder='little', signed=True) 
        slave1_data2 = int.from_bytes(slave1.input[4:8], byteorder='little', signed=False) 
        slave1_data3 = int.from_bytes(slave1.input[8:10], byteorder='little', signed=True) 
        slave1_data4 = int.from_bytes(slave1.input[10:12], byteorder='little', signed=False) 
        slave1_data5 = ctypes.c_int8(slave1.input[12]).value
        slave1_data6 = slave1.input[13]
        textboxes2[0].delete(0, 'end')
        textboxes2[0].insert(0, slave1_data1)
        textboxes2[1].delete(0, 'end')
        textboxes2[1].insert(0, slave1_data2)
        textboxes2[2].delete(0, 'end')
        textboxes2[2].insert(0, slave1_data3)
        textboxes2[3].delete(0, 'end')
        textboxes2[3].insert(0, slave1_data4)
        textboxes2[4].delete(0, 'end')
        textboxes2[4].insert(0, slave1_data5)
        textboxes2[5].delete(0, 'end')
        textboxes2[5].insert(0, slave1_data6)
        time.sleep(0.1)

ethercat_thread = threading.Thread(target=thread1, daemon=True)
ethercat_thread.start()

root.mainloop()

print("프로그램이 종료되었습니다.")
master.state = pysoem.INIT_STATE
# request INIT state for all slaves
master.write_state()
master.close()
